@@ -51,3 +51,54 @@ calcStandardErrors <- function(C,
   se <- sqrt(x)
   return(se)
 }
+
+#' Test function for predict, for the moment internal
+#'
+#' @keywords internal
+predictTest <- function(object,
+                        classify) {
+  term.labels <- c(object$term.labels.f, object$term.labels.r)
+
+  isFixed <- term.labels %in% object$term.labels.f
+
+  dim <- object$dim
+
+  e <- cumsum(dim)
+  s <- e - dim + 1
+  df.terms <- data.frame(label = term.labels, s = s, e = e, fixed = isFixed)
+
+  f <- which(classify == term.labels)
+
+  ndxPred <- object$ndxCoefficients[[classify]]
+
+  nr <- length(ndxPred)
+  nc <- ncol(object$C)
+  Dg <- spam::spam(x = 1, nrow = nr, ncol = 1) # intercept.
+  for (i in 2:length(df.terms)) {
+    if (i == f) {
+      M <- spam::diag.spam(nr)
+      if (df.terms$fixed[i]) {
+        M <- M[, -1]
+      }
+    } else {
+      if (df.terms$fixed[i]) {
+        M <- spam::spam(x = 1 / (dim[i] + 1), nrow = nr, ncol = dim[i])
+      } else {
+        M <- spam::diag.spam(x = 0, nrow = nr, ncol = dim[i])
+      }
+    }
+    Dg <- spam::cbind.spam(Dg, M)
+  }
+  cat("Dim Dg", dim(Dg),"\n")
+
+  ypred <- as.vector(Dg %*% object$coefMME)
+  ypredse <- calcStandardErrors(object$C, Dg)
+
+  df <- data.frame(name=names(ndxPred), prediction = ypred, se = ypredse)
+  return(df)
+}
+
+
+
+
+
